/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx_api.h	 
 *
 *	API definitions
 *
 ************************************************************************/
/*
 *
 *  15-Feb-94 -- Cornu
 *      Creation
 */

#ifndef		WDX_API
#define		WDX_API

/* application allowed operations */
#define WDX_GROUP_RUN        10 /* run operation */
#define	WDX_GROUP_KILL       11 /* kill operation */
#define	WDX_GROUP_STOP       12 /* stop operation */
#define	WDX_GROUP_START      13 /* start operation */
#define	WDX_GROUP_EXCEPTION  14 /* exception operation */
#define	WDX_MEMBER_UP        15 /* join operation */
#define WDX_PROCESS_KILL     16 /* leave operation */
#define	WDX_GROUP_PAUSE      17 /* pause operation */

#define	WDX_WAIT_FOR_RESERVE 20 /* waitreserve operation */
#define	WDX_WAIT_FOR_MOUNT   21 /* waitmount operation */

#define	WDX_LOG	             30 /* log operation */
#define	WDX_CONSTANT         31 /* version operation */
#define	WDX_LIST	     32 /* list operation */
#define	WDX_MASTERNAME       33 /* mastername operation */
#define	WDX_STATUS	     34 /* status operation */
#define	WDX_LOCALNAME        35 /* local name operation */
#define WDX_SCRIPT           36 /* constant operation second part */

#define WDX_DECLARE          40 /* declare operation */
#define	WDX_ERASE	     41 /* erase operation */
#define	WDX_RESERVE	     42 /* reserve operation */
#define	WDX_MOUNT	     43 /* mount operation */
#define	WDX_FREE             44 /* free operation */
#define	WDX_FAULT	     45 /* fault operation */
#define	WDX_GETMASTER        46 /* get master operation */
#define	WDX_GET              47 /* get ressource operation */
#define	WDX_MASTER	     48 /* master operation */
#define WDX_NOMASTER         49 /* nomaster operation */

/* Mode */
enum wdx_api_mode { REQUEST_ONLY,
		    EVENT_ONLY,
		    REQUEST_AND_EVENT };

/* Routines prototypes */
int wdx_open(char *missionname,int mode);
int wdx_close(int descriptor);
int wdx_write(int descriptor,int type,char *string,int arg,int *status,char *message,int size);
int wdx_read(int descriptor,int *type,int *status,int *identifier,int *generation,int *rstatus,char *message,int length);

/* Error returned by API */
#define WDX_INVAL 1
#define WDX_NOBUFS 2
#define WDX_ACCES 3
#define WDX_IO 4
#define WDX_NOTRUN 5
#define WDX_NOTCONN 6
#define WDX_PERM 7
#define WDX_EXIST 8
#define WDX_SLCK 9
#define WDX_LKLCK 10

#define WDX_MAX_API_STATUS_LENGTH 40
#define WDX_MAX_ERROR_NUMBER (WDX_LKLCK+1)
#ifdef WDX_API_MODULE
char  errortable[WDX_MAX_ERROR_NUMBER][WDX_MAX_API_STATUS_LENGTH] =
 { "Not used",
   "Invalid argument",
   "No buffer space available",
   "Permission denied",
   "Communication error",
   "Wdx process not started",
   "Can't connect to wdx process",
   "Command not allowed to satellite",
   "Connection already exist",
   "Cannot set lock",
   "Cannot look at the other lock"
 };
#else
extern char errortable[WDX_MAX_ERROR_NUMBER][WDX_MAX_API_STATUS_LENGTH];
#endif
#endif	/* WDX_API */
