#ifndef lint
char decquality_id[] = "@(#)wdx 0.1   15/07/03";
#endif
/*
 *   WDX: WitchDoctor-WDX, versatile distributed lock manager
 *   Copyright (C) 2003 Dominique Chabord,
 *   Please refer to credits section below
 *
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   ************************************************
 *
 *   History and credits:
 *   First version of WitchDoctor-WDX was based on:
 *
 *   DECwatchdog/Autopilot, watchdog and lock manager
 *   Copyright (C) 2003 HP
 *
 *   Developed by
 *     Computer Special Systems, Annecy France
 *     A Division of Digital Equipment Corporation
 *   and
 *     Custom-Systems, Annecy France
 *     An engineering center of Compaq
 *
 *   Provided under GNU-GPL terms by HP, 30 June 2003
 *   GNU-GPL replaces previous license statements
 *
 *   Rebranded, ported and modified for Shaman-X project, 15 July 2003
 *   by Dominique Chabord
 *   and maintained by other contributors as mentionned in the code
 *
 *   ************************************************
 *
 *   WDX is promoted by Shaman-X project
 *
 *   Check for new versions and documentation at
 *   http://www.shaman-x.org
 *
 *   ************************************************
 *
 *   Most of WDX code is designed to run unchanged on
 *   different operating systems, including at V1.0 time:
 *   Gnu-Linux, WindowsNT, DigitalUNIX and OpenVMS.
 *   Operating system specifics are identified by dedicated files.
 *
 *   Evolutions
 *   ----------
 *   July-2003     Dominique Chabord    WitchDoctor-WDX V1.0
 *                 Creation
 *
 *   ************************************************
 *
 *
 *	wdx_ui.c
 *
 *	User interface and main module
 *
 *      Routines :
 *
 *      main()
 *      int daemoncontrol()
 *      int resourcecontrol()
 *      int resourcecontrol2()
 *      int supercontrol()
 *	int snapcontrol()
 *      int noargcontrol()
 *      int erroroption()
 *      void print_usage()
 *      int process_show()
 ************************************************************************/
/*
 *
 *  05-Mar-94 -- Cornu
 *      Creation
 */


#undef  WDX_UI_MODULE
#define WDX_UI_MODULE

#define MAIN

#include	"wdx.h"
#include	"wdx_ui.h"
#include	"wdx_config.h"

#define BADARG    1
#define GOODARG   0

char        *rpt = "";             /* input resource name pointer */
int        second_nb = 0;              /* wanted pause duration */
char       mission[MISSION_LEN];   /* mission name */
FILE      *outdes;	           /* output descriptor */
extern int wdxerrno;

/* Internal references */
int ui_parse(char *val1,char *val2);
void print_usage();
void print_error_message(int e);
int one_command(enum wdx_type type,int command,FILE *logdes);

main(argc,argv)
int argc;
char *argv[];
{
struct userinterface      *cmd; /* user interface pointer */
int                  verbcount; /* number of verbs in command line */
int             ambiguitycount; /* ambiguity for a verb */
int                    retcode; /* code returned by option function */
enum wdx_type             type; /* store found option type */
int                    command; /* to store command */
char		       **value; /* pt to value of a qualifier */
int               (*control)(); /* pt to control function pointer */
int                (*action)(); /* pt to action function pointer */
/******************************************************************************/
/* Process init
/******************************************************************************/
    pn = spc_get_process_name(argv[0]); /* make program name globally avail. */
/*-- delete LIONEL TRICON --*/
    /*if ( spc_probe_license() == -1 )
         exit(WDX_FATAL_ERROR);
    atexit(spc_release_license);*/
/*-- fin delete LIONEL TRICON --*/
    outdes = stdout;

/* if the program name is not "wdx" then it is in test mode and will be
   using IP addresses as well as broadcasts for transmitting to the network
*/
	if (strcmp (pn, DEFAULT_PROG_NAME) == 0)
		using_ip_addresses = 0;
	else
		using_ip_addresses = 1;

/******************************************************************************/
/* Get mission name
/******************************************************************************/
    argc--;
    argv++;
    memset(mission,'\0',MISSION_LEN);
    strcpy(mission,DEFAULT_MISSION);
    if ( argc > 0 )
         if ( *argv[0] != WDX_SEPARATOR )
         {
              if ( strlen(*argv) >= MISSION_LEN )
	      {
	           PERROR_B
	           "Mission name is too long"
	           PERROR_E;
		   exit(WDX_FATAL_ERROR);
	      }
              strncpy(mission,*argv,MISSION_LEN);
              argc--;
              argv++;
         }
/******************************************************************************/
/* Parse qualifiers
/******************************************************************************/
    verbcount = 0;
    while(argc > 0 )
    {
	/* search for option keyword */
	ambiguitycount = 0;
        for(cmd=ui;cmd->option;cmd++)
           if(ui_parse(*argv,cmd->option) == 0 )
        {
               ambiguitycount++;
	       verbcount++;
	       value = argv +1;
               type = cmd->type;
               command = cmd->command;
	       control = cmd->controlfunct;
	       action = cmd->actionfunct;
        }
	/* single option supported */
        if ( ambiguitycount > SINGLEOPTION )
        {
	     PERROR_B
	     "Ambiguous qualifier, type more characters"
	     PERROR_E;
             exit(WDX_FATAL_ERROR);
        }
        argc--; /* argv forward */
        argv++;
    }
    if ( verbcount > SINGLEOPTION )
    {
	PERROR_B
	"Multiple qualifiers not allowed"
	PERROR_E;
	exit(WDX_FATAL_ERROR);
    }
    if ( verbcount != SINGLEOPTION )
    {
	print_usage();
        exit(WDX_FATAL_ERROR);
    }
/******************************************************************************/
/* Get and check qualifier values
/******************************************************************************/
    if(control)
    {
           if ((retcode = (*control)(*value)) < 0)
		exit(WDX_FATAL_ERROR);
           else if ( retcode == BADARG ) /* no argument passed : argv back */
           {
                print_usage();
                exit(WDX_FATAL_ERROR);
           }
    }
/******************************************************************************/
/* One single option successfully decoded : process the operation
/******************************************************************************/
    retcode = 0;
    if(action)
       exit((*action)(type,command,outdes));
}

/* ui_parse()
 * Parse val1 into val2
 *
 * Return 0 if ok
 */
int ui_parse(char *val1,char *val2)
{
char *p1;
char *p2;
int stop;

   stop = 0;
   p1 = val1;
   p2 = val2;
   while( *p1 && *p2 && !stop )
   {
	if ( toupper(*p1) != toupper(*p2) )
 	     stop++;
        p1++;
	p2++;
   }
   if ( !stop )
        return(0);
   else return(1);
}

/* int ui_daemoncontrol()
 * Start daemon option :
 *      if argument following -daemon does not define an option
 *              local member name is argument
 *      else    local member name is local TCP/IP hostname
 *
 * Argument : pt to argument
 *
 * Return   : -1 if error / GOODARG if OK
 */
int ui_daemoncontrol(p)
char *p;
{

   if ( ui_supercontrol(p) == -1 )
        return(-1);
   memset(localname,'\0',sizeof(localname));
   if(p)
   {
       if ( strlen(p) >= sizeof(localname) )
       {
	    PERROR_B
	    "Member name is too long"
	    PERROR_E;
	    return(-1);
       }
       strncpy(localname,p,sizeof(localname));
   }
   else if (gethostname(localname,sizeof(localname)-1) < 0)
   {
       PERROR_B
       "Can't get hostname"
       PERROR_E;
       return(-1);
   }
   return(GOODARG);
}

/* int ui_resourcecontrol()
 * int ui_resourcecontrol2()
 * Resource or directory management option :
 *      if argument following the option does not define an option
 *              rpt is argument
 *      else    error
 * resourcecontrol() also checks if efective user id is 0 ( super user )
 *
 * Argument : pt to next argument
 *
 * Return   : -1 : error / BADARG or GOODARG
 */
int ui_resourcecontrol(p)
char *p;
{
   if ( ui_supercontrol(p) == -1 )
        return(-1);

/* DM */
/*   ui_resourcecontrol2(p);     */

   if(p)
   {
        if ( strlen(p) >= RESOURCE_LEN )
        {
             PERROR_B
             "Name of resource is too long"
             PERROR_E;
             return(-1);
        }
        rpt = p;
        return(GOODARG);
   }
   return(BADARG);
}

int ui_resourcecontrol2(p)
char *p;

{
   if(p)
   {
        if ( strlen(p) >= RESOURCE_LEN )
        {
             PERROR_B
             "Name of resource is too long"
             PERROR_E;
             return(-1);
        }
        rpt = p;
        return(GOODARG);
   }
   return(BADARG);
}

/* int ui_supercontrol()
 * Verify that user belongs to superuser group
 *
 * Argument : None
 *
 * Return   : GOODARG : ok / -1 error
 */
int ui_supercontrol(char *p)
{
   if ( spc_super_user() == 0 )
   {
        PERROR_B
        "Command restricted to superuser"
        PERROR_E;
        return(-1);
   }
   return(GOODARG);
}

/* int ui_noargcontrol()
 * No argument excepted function.
 *
 */
int ui_noargcontrol(char *p)
{
   return(GOODARG);
}

/* int ui_snapcontrol()
 * Resource or directory management option :
 *      if argument following the option does not define an option
 *              rpt is argument
 *      else    error
 *
 */
int ui_snapcontrol(char *p)
{

   if ( ui_supercontrol(p) == -1 )
        return(-1);
   if(p)
   {
        if ( strlen(p) >= SNAPSHOT_LEN )
        {
             PERROR_B
             "Name of snapshot is too long"
             PERROR_E;
             return(-1);
        }
        rpt = p;
        return(GOODARG);
   }
   else rpt = mission;
   return(GOODARG);
}

/* int ui_pausecontrol()
 * Verify the user is SuperUser
 * Verify the argument given for the pause command is an integer with a
 * value betweeen 1 to 2**32-1
 *      else    error
 *
 */
int ui_pausecontrol(char *p)
{

   if ( ui_supercontrol(p) == -1 )
        return(-1);
   if(p)
   {
    if ( lib_is_numeric(p) )
        {
        PERROR_B
        "Incorrect number of seconds"
        PERROR_E;
        return(BADARG);
        }
   second_nb = atoi(p);
   if ( second_nb < 1 )
        {
        PERROR_B
        "Incorrect number of seconds"
        PERROR_E;
        return(BADARG);
        }
   rpt = mission;
   return(GOODARG);
   }
   else
   {
     PERROR_B
     "Number of seconds must be specified"
     PERROR_E;
     return(BADARG);
   }

}

int ui_errorcontrol(char *p)
{
   print_usage();
   return(-1);
}

int ui_logfile(char *p)
{
   if(p)
   {
	outdes = fopen(p,"w");
	if( outdes == NULL )
	{
	    sprintf(messagestring,"Can't open output file %s",p);
	    PERROR_B
	    messagestring
	    PERROR_E;
	    return(-1);
	}
   }
   return(GOODARG);
}

/* print_usage()
 * Print process usage using definition given in userinterface structure
 *
 * Argument : none
 *
 * Return   : none
 */
void print_usage()
{
struct userinterface *i;

    printf("Version 0.4 - Usage of \"%s\":\n",pn);
    for (i=ui;i->option;i++)
         printf(i->usage,pn);
}

/******************************************************************************/
/* Specific routines
/******************************************************************************/
/* ui_application()
 * Application must connect to wdx process for command processing
 * Display status.
 *
 * Argument : type of command
 *	      command value
 *
 * Return   : Exit value
 */
int ui_application(enum wdx_type type,int command,FILE *logdes)
{
int ret;

    spc_path_init(&wdxdirectory);	    /* setup wdx path */
    ret = one_command(type,command,logdes);
    if ( logdes != stdout )
	fclose(logdes);
    return(ret);
}

/* int ui_constant()
 * Get mission constants
 */
int ui_constant(enum wdx_type type,int command,FILE *logdes)
{
int ret;

   spc_path_init(&wdxdirectory);	    /* setup wdx path */
   ret = one_command(type,command,logdes);
   if ( ret != WDX_RETURN_SUCCESS )
        return(ret);
   ret = one_command(type,WDX_SCRIPT,logdes);
    if ( logdes != stdout )
	fclose(logdes);
   return(ret);
}

/* int ui_help()
 * Call spc_help routine to display on line help
 *
 * Arguments : standards for ui_routine
 *
 *
 * Return    : -1 if error
 *             0 if OK
 */
int ui_help(enum wdx_type type, int command , FILE *logdes)
{
int ret;

    ret = spc_help(logdes);
    if ( logdes != stdout )
	fclose(logdes);
    return(ret);
}

/* int ui_stop()
 * Stop a mission and create a .txt file
 *
 * Argument :
 *
 * Return   : 0  ok / -1 error
 */
int ui_stop(enum wdx_type type,int command,FILE *logdes)
{
int			   des; /* api descriptor */
int			    st; /* status */
char display[PROCESS_DATA_LEN];
char filename[STRING_LEN];
FILE *fdes;
time_t  generation;

    des = -1;
    spc_path_init(&wdxdirectory);
/* create ASCII file name */
    sprintf(filename,SNAPASCII_FILE,wdxdirectory,mission,rpt);
    if ( (fdes = fopen(filename,"w+")) == NULL )
    {
	PERROR_B
	"Can't create the snapshot documentation file "
	PERROR_E;
	return(WDX_RETURN_ERROR);
    }

/* insert loc file contents into snapshot ASCII */
/*    fprintf(fdes,"\n%s\n%s%s\n%s\n\n\n%s\n%s\n%s\n",
/*	    SNAPSHOT_INTERVAL,
/*	    SNAPSHOT_TITLE,
/*	    lib_generation_to_s(&generation),
/*	    SNAPSHOT_INTERVAL,
/*	    SNAPSHOT_INTERVAL,
/*	    SNAPSHOT_CFG,
/*	    SNAPSHOT_INTERVAL);
/*    sprintf(fileread,MISSION_FILE,wdxdirectory,mission);
/*    if ( (fread = fopen(fileread,"r")) == NULL )
/*    {
/*
/*	PERROR_B
/*	"Can't open mission file"
/*	PERROR_E;
/*	fclose(des);
/*	spc_unlink(filename);
/*	return(WDX_RETURN_ERROR);
/*    }
/*    while(fgets(buffer,sizeof(buffer),fread) != NULL )
/*	fputs(buffer,des);
/*    fclose(fread);
/*
/*
/*    fprintf(fdes,"\n%s\n%s\n%s\n",
/*	    SNAPSHOT_INTERVAL,
/*	    SNAPSHOT_LOC,
/*	    SNAPSHOT_INTERVAL);
/*    sprintf(fileread,LOCAL_FILE,wdxdirectory,mission);
/*    if ( (fread = fopen(fileread,"r")) == NULL )
/*    {
/*	PERROR_B
/*	"Can't open local file"
/*	PERROR_E;
/*	fclose(des);
/*	spc_unlink(filename);
/*	return(WDX_RETURN_ERROR);
/*    }
/*    while(fgets(buffer,sizeof(buffer),fread) != NULL )
/*	fputs(buffer,des);
/*    fclose(fread);
*/
/* connect to daemon process */
    des = wdx_open(mission,REQUEST_ONLY);
    if ( des == -1 )
    {
	print_error_message(wdxerrno);
	fclose(fdes);
	spc_unlink(filename);
	return(WDX_RETURN_ERROR);
    }

/* get list information */
    if ( ( wdx_write(des,
		     WDX_LIST,
		     "",
                     0,
		     &st,
	 	     display,
	 	     sizeof(display))) == -1 )
    {
	print_error_message(wdxerrno);
	fclose(fdes);
	spc_unlink(filename);
        wdx_close(des);
	return(WDX_RETURN_ERROR);
    }
    generation = lib_build_generation();
    fprintf(fdes,"\n%s\n%s%s\n%s\n\n\n%s\n%s\n%s\n%s\n",
	    SNAPSHOT_INTERVAL,
	    SNAPSHOT_TITLE,
	    lib_generation_to_s(&generation),
	    SNAPSHOT_INTERVAL,
	    SNAPSHOT_INTERVAL,
	    SNAPSHOT_LIST,
	    SNAPSHOT_INTERVAL,
	    display);

/* get constant and version information */
    if ( ( wdx_write(des,
		     WDX_CONSTANT,
		     "",
                     0,
		     &st,
	 	     display,
	 	     sizeof(display))) == -1 )
    {
	print_error_message(wdxerrno);
	fclose(fdes);
	spc_unlink(filename);
        wdx_close(des);
	return(WDX_RETURN_ERROR);
    }
    fprintf(fdes,"%s\n%s\n%s\n%s\n",
	    SNAPSHOT_INTERVAL,
	    SNAPSHOT_CONSTANT,
	    SNAPSHOT_INTERVAL,
	    display);
    if ( ( wdx_write(des,
		     WDX_SCRIPT,
		     "",
                     0,
		     &st,
	 	     display,
	 	     sizeof(display))) == -1 )
    {
	print_error_message(wdxerrno);
	fclose(fdes);
	spc_unlink(filename);
        wdx_close(des);
	return(WDX_RETURN_ERROR);
    }
    fprintf(fdes,"%s\n",
	    display);
    fclose(fdes);
/* stop command */
    if ( ( wdx_write(des,
		     command,
		     rpt,
                     second_nb,
		     &st,
	 	     display,
	 	     sizeof(display))) == -1 )
    {
	print_error_message(wdxerrno);
	spc_unlink(filename);
        wdx_close(des);
	return(WDX_RETURN_ERROR);
    }
    display[sizeof(display)-1] = '\0';
    printf("%s\n",display);
    wdx_close(des);
    return(WDX_RETURN_SUCCESS);
}

/* ui_daemon()
 * create a daemon process after initialization.
 *
 * Argument : Not used
 *
 * Return   : Exit value
 */
int ui_daemon(enum wdx_type type,int command,FILE *logdes)
{
int retcode; /* returned code */

//printf("ui_daemon spc_create_daemon ker_scheduler\n");
   retcode = spc_create_daemon(mission,
				&ggs,
				&message,
				ker_scheduler);
   return(retcode);
}

/* int ui_show()
 * Process a show command
 *
 * Argument : wdx_type : NOT USED
 *            command  : NOT USED
 *
 *
 */
int ui_show(enum wdx_type type,int command,FILE *logdes)
{
struct mission_list *head;	/* head of mission list queue */
struct mission_list *current;   /* current element in queue */
int                 enteteflag; /* flag used by WDX_SHOW option */
char      master[MEMBER_LEN+1]; /* master name */
char        name[MEMBER_LEN+1]; /* local name */
int			   des; /* api descriptor */
char       format1[STRING_LEN];
int			    st; /* status */
int                       step; /* step */
int                        ret; /* returned value */

#define SHOW_OPEN 0
#define SHOW_WRITE 1
#define SHOW_CLOSE 2
#define SHOW_NEXT 3
#define SHOW_STOP 4
#define SHOW_GET 5
#define SHOW_PRINT 6

    enteteflag = 0;
    des = -1;
    spc_path_init(&wdxdirectory);
    step = SHOW_GET;
    ret = WDX_RETURN_SUCCESS;
    while(step != SHOW_STOP)
    {
	switch(step)
	{
	case SHOW_GET :
            head = spc_get_running_mission();
            if ( head == (struct mission_list *) NULL )
	    {
	         step = SHOW_STOP;
		 ret = WDX_FATAL_ERROR;
	    }
	    else
	    {
                 step = SHOW_OPEN;
		 current = head;
	    }
	    break;
	case SHOW_OPEN :
            des = wdx_open(current->mission,REQUEST_ONLY);
            if ( des == -1 )
		 step = SHOW_NEXT;
	    else step = SHOW_WRITE;
	    break;
	case SHOW_WRITE :
            if ( ( wdx_write(des,
		             WDX_MASTERNAME,
		             "",
                             0,
		             &st,
	 	             master,
	 	             sizeof(master))) == -1 )
		 step = SHOW_CLOSE;
            else if ( ( wdx_write(des,
		             WDX_LOCALNAME,
		             "",
                             0,
		             &st,
	 	             name,
	 	             sizeof(name))) == -1 )
		 step = SHOW_CLOSE;
            else step = SHOW_PRINT;
	    break;
	case SHOW_PRINT :
            if ( !enteteflag )
            {
                 fprintf(logdes,"\nLocal processing missions :\n\n");
                 sprintf(format1,
                         "%%-%ds  %%-%ds  %%-%ds\n",
                         MISSION_FORMAT,
                         MEMBER_FORMAT,
                         MEMBER_STATUS_FORMAT);
                 fprintf(logdes,format1,"MISSION NAME",
                         "LOCAL MEMBER NAME","MASTERHOLDER");
            }
            enteteflag = 1;
	    if ( strcmp(master,MASTERHOLDER_IS_UNKNOWN) == 0 )
                 fprintf(logdes,format1,current->mission,name,"");
            else fprintf(logdes,format1,current->mission,name,master);
	    step = SHOW_CLOSE;
	    break;
        case SHOW_CLOSE :
	    if ( des != -1 )
	         wdx_close(des);
	    step = SHOW_NEXT;
	    break;
        case SHOW_NEXT :
	    current = current->next;
	    if (!current)
	    {
		 current = head;
		 while(current)
		 {
		    head = current->next;
		    free(current);
		    current = head;
		 }
		 step = SHOW_STOP;
	    }
            else step = SHOW_OPEN;
	    break;
	}
    }
    if ( !enteteflag )
    {
        fprintf(logdes,"No mission currently processing\n");
    }
    if ( logdes != stdout )
	fclose(logdes);
    return(ret);
}

/* void print_error_message()
 * Print an error message returned by the API
 *
 * Argument : error code
 *
 * Return   : None
 */
void print_error_message(int e)
{

   PERROR_B
   &errortable[e][0]
   PERROR_E;
}

/* one_command()
 * Application must connect to wdx process for command processing
 * Display status.
 *
 * Argument : type of command
 *	      command value
 *
 * Return   : Exit value
 */
int one_command(enum wdx_type type,int command,FILE *logdes)
{
int  des;
char display[PROCESS_DATA_LEN];
enum process_status st;
enum resource_status rs;
enum process_type ty;
int               id1;
int               id2;
int               ge;


    des = wdx_open(mission,REQUEST_ONLY);
    if ( des == -1 )
    {
	 print_error_message(wdxerrno);
         return(WDX_FATAL_ERROR);
    }
    if ( ( id1 = wdx_write(des,
		   command,
		   rpt,
                   0,
		   (int *)&st,
		   display,
		   sizeof(display))) == -1 )
    {
	 print_error_message(wdxerrno);
	 return(WDX_FATAL_ERROR);
    }
    else if ( st == PROCESS_SUCCESS )
    {

         switch(type)
         {
         case WDX_COMMAND :
             break;
         case WDX_REQUEST :
     	     do
             {

                 if ( wdx_read(des,
                	       (int *)&ty,
	          	       (int *)&st,
	   	            &id2,
		            &ge,
          	            (int *)&rs,
     	 	            display,
		            sizeof(display)) == -1 )
		 {
	              print_error_message(wdxerrno);
                      return(WDX_FATAL_ERROR);
                 }
             }
	     while((id2 != id1) || ( ty == PROCESS_CONTROL ) );

	     break;
         }
    }

    /* display is redirected to logdes file */
    display[sizeof(display)-1] = '\0';
    fprintf(logdes,"%s\n",display);
    wdx_close(des);

    switch(st)
    {
    case PROCESS_SUCCESS :
         return(WDX_RETURN_SUCCESS);
         break;
    default :
	 if ( command == WDX_MASTER && wdxerrno == WDX_PERM )
	      return(WDX_RETURN_SATELLITE);
         return(WDX_RETURN_ERROR);
	 break;
    }
}
